package cz.jzitnik.chronos.controllers;

import cz.jzitnik.chronos.entities.Item;
import cz.jzitnik.chronos.entities.Message;
import cz.jzitnik.chronos.entities.MessageType;
import cz.jzitnik.chronos.entities.Room;
import cz.jzitnik.chronos.payload.errors.NotFoundError;
import cz.jzitnik.chronos.payload.responses.UnifiedResponse;
import cz.jzitnik.chronos.repository.PlayerRepository;
import cz.jzitnik.chronos.repository.RoomRepository;
import cz.jzitnik.chronos.services.GameService;
import cz.jzitnik.chronos.services.ItemService;
import cz.jzitnik.chronos.utils.anotations.CheckUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/game/rooms")
public class RoomController {
    @Autowired
    private PlayerRepository playerRepository;

    @Autowired
    private RoomRepository roomRepository;

    @Autowired
    private GameService gameService;

    @Autowired
    private ItemService itemService;

    @GetMapping
    @CheckUser
    public ResponseEntity<UnifiedResponse<List<Room>, Error>> getAllRooms(@RequestParam String playerKey) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        return ResponseEntity.ok(
                UnifiedResponse.success(player.getGame().getRooms())
        );
    }

    @GetMapping("/room")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Room, Error>> getRoom(@RequestParam String playerKey, @RequestParam Long roomId) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var roomOptional = roomRepository.findById(roomId);

        if (roomOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(
                    UnifiedResponse.failure(new NotFoundError("Room with id " + roomId + " was not found!"))
            );
        }

        var room = roomOptional.get();

        if (!room.getGame().getId().equals(player.getGame().getId())) {
            return ResponseEntity.status(HttpStatus.FORBIDDEN).body(
                    UnifiedResponse.failure(new Error("This room is from a different game!"))
            );
        }

        return ResponseEntity.ok(UnifiedResponse.success(room));
    }

    @GetMapping("/current_room")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Room, Error>> getCurrentRoom(@RequestParam String playerKey) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        return ResponseEntity.ok(
                UnifiedResponse.success(player.getCurrentRoom())
        );
    }

    @PostMapping("/move")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Object, Error>> moveToRoom(@RequestParam String playerKey, @RequestParam Long roomId) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var roomOptional = roomRepository.findById(roomId);

        if (roomOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(UnifiedResponse.failure(null));
        }

        var room = roomOptional.get();

        var fromThisGame = gameService.isFromThisGame(player.getGame(), room);

        if (!fromThisGame) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(UnifiedResponse.failure(new Error("Invalid roomId")));
        }

        player.setCurrentRoom(room);

        // Send message to chat
        var message = new Message(player, String.valueOf(roomId), MessageType.MOVE_TO_ROOM);
        player.getMessages().add(message);
        playerRepository.save(player);

        return ResponseEntity.ok(UnifiedResponse.success(null));
    }

    @PostMapping("/take_items")
    @CheckUser
    public ResponseEntity<UnifiedResponse<List<Item>, Error>> takeItems(@RequestParam String playerKey) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var room = player.getCurrentRoom();

        if (room.getItems().isEmpty()) {
            return ResponseEntity.status(HttpStatus.NO_CONTENT).body(UnifiedResponse.failure(new Error("No items in this room!")));
        }

        var items = room.getItems();

        var availableSlots = player.getInventorySize() - player.getInventory().size();
        System.out.println(availableSlots);

        var playerGotItems = new ArrayList<Item>();

        for (var i = 0; i <= Math.min(items.size(), availableSlots) - 1; i++) {
            var item = items.get(i);
            item.setRoom(null);
            item.setOwner(player);
            itemService.addItem(player, item);
            playerGotItems.add(item);
        }

        return ResponseEntity.ok(UnifiedResponse.success(playerGotItems));
    }
}
